package com.totopia.server.init;

import com.totopia.server.modules.dashboard.repository.DashboardRepository;
import com.totopia.server.modules.game.entity.LeagueEntity;
import com.totopia.server.modules.game.repository.LeagueRepository;
import com.totopia.server.modules.game.type.SportsTypeName;
import com.totopia.server.modules.site.entity.BankAccountsEntity;
import com.totopia.server.modules.site.repository.BankAccountsRepository;
import com.totopia.server.modules.user.entity.GradeEntity;
import com.totopia.server.modules.user.entity.RoleEntity;
import com.totopia.server.modules.user.entity.UserEntity;
import com.totopia.server.modules.user.repository.GradeRepository;
import com.totopia.server.modules.user.repository.RoleRepository;
import com.totopia.server.modules.user.repository.UserRepository;
import com.totopia.server.modules.user.type.GradeName;
import com.totopia.server.modules.user.type.RoleName;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;

import java.util.HashSet;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Component
@ConditionalOnProperty(name = "app.db-init", havingValue = "true")
@Slf4j
public class DbInitializer implements CommandLineRunner {

  @Autowired
  private RoleRepository roleRepository;

  @Autowired
  private UserRepository userRepository;

  @Autowired
  PasswordEncoder passwordEncoder;

  @Autowired
  DashboardRepository dashboardRepository;

  @Autowired
  GradeRepository gradeRepository;

//  @Autowired
//  SportsRepository sportsRepository;

  @Autowired
  LeagueRepository leagueRepository;

  @Autowired
  BankAccountsRepository bankInfoRepository;

  @Override
  public void run(String... strings) throws Exception {
    if (0 == roleRepository.count()) {
      this.initDbRole();
    }

    if (0 == gradeRepository.count()) {
      this.initDbGrade();
    }

//    if (0 == sportsRepository.count()) {
//      this.initDbSports();
//    }
    if (0 == userRepository.count()) {
      this.initDbUsers();
    }

    if (0 == leagueRepository.count()) {
      this.initDbLeague();
    }

    if (0 == bankInfoRepository.count()) {
      this.initDbBankInfo();
    }
    log.debug("-- Database has been initialized");
  }

  private void initDbRole() {
    RoleEntity role;
    role = RoleEntity.builder().name(RoleName.ROLE_USER).build();
    roleRepository.save(role);
    role = RoleEntity.builder().name(RoleName.ROLE_TEAM).build();
    roleRepository.save(role);
    role = RoleEntity.builder().name(RoleName.ROLE_AGENCY).build();
    roleRepository.save(role);
    role = RoleEntity.builder().name(RoleName.ROLE_ADMIN).build();
    roleRepository.save(role);
    role = RoleEntity.builder().name(RoleName.ROLE_SUPER_ADMIN).build();
    roleRepository.save(role);
  }

  private void initDbUsers() {
    UserEntity user;
    user = UserEntity.builder().username("admin").password(passwordEncoder.encode("admin")).nickname("admin")
            .email("admin@example.com").block(false).resetCount(0L).sendEmail(true).roles(Stream.of(RoleEntity.builder().id(Short.valueOf((short) 1)).build())
                    .collect(Collectors.toCollection(HashSet::new)))
            .build();

    userRepository.save(user);
    user = UserEntity.builder().username("test1").password(passwordEncoder.encode("qwer5795")).nickname("test1")
            .email("test1@example.com").block(false).resetCount(0L).sendEmail(true).roles(Stream.of(RoleEntity.builder().id(Short.valueOf((short) 2)).build())
                    .collect(Collectors.toCollection(HashSet::new)))
            .build();
    userRepository.save(user);

    user = UserEntity.builder().username("test2").password(passwordEncoder.encode("qwer5795")).nickname("test2")
            .email("test1@example.com").block(false).resetCount(0L).sendEmail(true).roles(Stream.of(RoleEntity.builder().id(Short.valueOf((short) 2)).build())
                    .collect(Collectors.toCollection(HashSet::new)))
            .build();

    userRepository.save(user);
  }
  private void initDbGrade() {
    GradeEntity grade;
    grade = GradeEntity.builder().name(GradeName.BAD_USER).build();
    gradeRepository.save(grade);
    grade = GradeEntity.builder().name(GradeName.NORMAL_USER).build();
    gradeRepository.save(grade);
    grade = GradeEntity.builder().name(GradeName.REGULAR_USER).build();
    gradeRepository.save(grade);
    grade = GradeEntity.builder().name(GradeName.VIP_USER).build();
    gradeRepository.save(grade);
    grade = GradeEntity.builder().name(GradeName.PLATINUM_USER).build();
    gradeRepository.save(grade);
  }

//  private void initDbSports() {
//    SportsTypeEntity sportsEntity;
//    sportsEntity = SportsTypeEntity.builder().name(SportsTypeName.SOCCER).build();
//    sportsRepository.save(sportsEntity);
//    sportsEntity = SportsTypeEntity.builder().name(SportsTypeName.BASEBALL).build();
//    sportsRepository.save(sportsEntity);
//    sportsEntity = SportsTypeEntity.builder().name(SportsTypeName.BASKET_BALL).build();
//    sportsRepository.save(sportsEntity);
//    sportsEntity = SportsTypeEntity.builder().name(SportsTypeName.E_SPORTS).build();
//    sportsRepository.save(sportsEntity);
//    sportsEntity = SportsTypeEntity.builder().name(SportsTypeName.HAND_BALL).build();
//    sportsRepository.save(sportsEntity);
//    sportsEntity = SportsTypeEntity.builder().name(SportsTypeName.TABLE_TENIS).build();
//    sportsRepository.save(sportsEntity);
//    sportsEntity = SportsTypeEntity.builder().name(SportsTypeName.VALLEY_BALL).build();
//    sportsRepository.save(sportsEntity);
//    sportsEntity = SportsTypeEntity.builder().name(SportsTypeName.TENNIS).build();
//    sportsRepository.save(sportsEntity);
//  }

  private void initDbLeague() {
    LeagueEntity leagueEntity;

    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Bundesliga I")
            .korName("분데스리가")
            .sevenMName("GER BI")
//            .sportsEntities(Stream.of(SportsTypeEntity.builder().id(Short.valueOf((short) 1)).build()).collect(Collectors.toCollection(HashSet::new)))
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);
    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Bundesliga II")
            .korName("분데스리가 2")
            .sevenMName("GER B2")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany 3.Liga")
            .korName("3.리가")
            .sevenMName("GER L3")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);
    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Regionalliga West")
            .korName("레기오날리가 웨스트")
            .sevenMName("GER RL W")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);
    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Regionalliga North")
            .korName("레기오날리가 노스")
            .sevenMName("GER RL N")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Regionalliga North East")
            .korName("레기오날리가 노스 이스트")
            .sevenMName("GER RL NE")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Oberliga Rheinland-Pfalz/Saar")
            .korName("오벨리가 Rheinland-Pfalz/Saar")
            .sevenMName("GER OL R")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Regionalliga Bayern")
            .korName("레기오날리가 바이에른")
            .sevenMName("GER RL B")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Oberliga Hamburg")
            .korName("오벨리가 함부르크")
            .sevenMName("GER OL H")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Oberliga Bayern North")
            .korName("오벨리가 바이에른 노스")
            .sevenMName("GER OL BN")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Oberliga Baden-Wuerttemberg")
            .korName("오벨리가 Baden-Wuerttemberg")
            .sevenMName("GER OL BW")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Oberliga Westfalen")
            .korName("오벨리가 Westfalen")
            .sevenMName("GER OL W")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Oberliga NOFV Sud")
            .korName("오벨리가 NOFV Sud")
            .sevenMName("GER OL NS")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("Germany")
            .korCountry("독일")
            .engName("Germany Bundesliga Women")
            .korName("우먼 분데스리가")
            .sevenMName("Germany BW")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("England")
            .korCountry("잉글랜드")
            .engName("Barclays Premier League")
            .korName("프리미어 리그")
            .sevenMName("Barclays PL")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("England")
            .korCountry("잉글랜드")
            .engName("England League 2")
            .korName("리그 2")
            .sevenMName("UK L2")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("England")
            .korCountry("잉글랜드")
            .engName("England National League")
            .korName("내셔널 리그")
            .sevenMName("UK NL")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("England")
            .korCountry("잉글랜드")
            .engName("England National League North")
            .korName("내셔널 리그 노스")
            .sevenMName("UK NLN")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("England")
            .korCountry("잉글랜드")
            .engName("England National League South")
            .korName("내셔널 리그 사우스")
            .sevenMName("UK NLS")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("England")
            .korCountry("잉글랜드")
            .engName("England Isthmian Premier Division")
            .korName("이스미안 프리미어 디비전")
            .sevenMName("Isthmian PD")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("England")
            .korCountry("잉글랜드")
            .engName("England Southern Premier League South")
            .korName("사우던 프리미어 리그 사우스")
            .sevenMName("England SPL S")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("England")
            .korCountry("잉글랜드")
            .engName("England Southern Premier League Central")
            .korName("사우던 프리미어 리그 센트럴")
            .sevenMName("England SPL C")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("England")
            .korCountry("잉글랜드")
            .engName("England Southern League Division One")
            .korName("사우던 리그 디비전 1")
            .sevenMName("UK SLD1")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("England")
            .korCountry("잉글랜드")
            .engName("England U23 Development League")
            .korName("유스 U23 디벨럽먼트 리그")
            .sevenMName("England U23 DL")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("England")
            .korCountry("잉글랜드")
            .engName("Lancashire Senior Cup")
            .korName("랭커셔 시니어 컵")
            .sevenMName("UK LS Cup")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

    leagueEntity = LeagueEntity.builder()
            .engCountry("England")
            .korCountry("잉글랜드")
            .engName("England FA WSL")
            .korName("우먼 FA 슈퍼리그")
            .sevenMName("FA WSL")
            .sportsType(SportsTypeName.SOCCER)
            .build();
    this.leagueRepository.save(leagueEntity);

  }

  private void initDbBankInfo() {
    BankAccountsEntity bankInfoEntity;

    bankInfoEntity = BankAccountsEntity.builder()
        .name("국민은행")
        .number("11111-11-11111")
        .holder("테스트1")
        .description("1번계좌")
        .build();
    bankInfoRepository.save(bankInfoEntity);

    bankInfoEntity = BankAccountsEntity.builder()
        .name("우리은행")
        .number("22222-22-22222")
        .holder("테스트2")
        .description("2번계좌")
        .build();
    bankInfoRepository.save(bankInfoEntity);

    bankInfoEntity = BankAccountsEntity.builder()
        .name("기업은행")
        .number("33333-33-33333")
        .holder("테스트3")
        .description("3번계좌")
        .build();
    bankInfoRepository.save(bankInfoEntity);

    bankInfoEntity = BankAccountsEntity.builder()
        .name("신한은행")
        .number("44444-44-444444")
        .holder("테스트4")
        .description("4번계좌")
        .build();
    bankInfoRepository.save(bankInfoEntity);

    bankInfoEntity = BankAccountsEntity.builder()
        .name("국민은행")
        .number("55555-55-55555")
        .holder("테스트5")
        .description("5번계좌")
        .build();
    bankInfoRepository.save(bankInfoEntity);
  }
}
